# -*- coding: utf-8 -*-

# Vendor Library
from spyne.decorator import rpc
from spyne.model.primitive import Unicode
from spyne.model.complex import Array

# ProjectLibrary
from kinder.webservice.api import declaration as decl_api
from kinder.webservice.api.exceptions import ApiException
from kinder.webservice.spyne_ws.exceptions import SpyneException

from kinder.webservice.spyne_ws.declaration_info.types import (
    DeclarantPersonal, ChildPersonal)
from kinder.webservice.spyne_ws.declaration_info import helpers
from kinder.webservice.spyne_ws.declaration_info.errors import (
    DeclCheckErrors,
    DeclCheckException
)
from lipetsk_specifics.webservice.entities import (
    NewApplicationRequest, GetApplicationResponse,
    UpdateApplicationRequest, UpdateApplicationResponse,
    DeclarationStatus)
from lipetsk_specifics.webservice.proxy import (
    LipetskNewApplicationProxy)
from spyne_objects_proxy import (
    LipetskDeclaredPersonDataProxy, LipetskApplicantDataProxy,
    LipetskEduOrganizationsDataProxy)
from .helpers import RegionalCheckDeclarationProcess, get_binary_data
from .config import cont
from .entities import NewApplicationResponse, GetApplicationRequest
from .proxy import GetApplicationProxy


# Методы концентратора
# - Передача Заявления в региональную систему предоставления услуг
#   NewApplication
@rpc(
    NewApplicationRequest,
    _returns=NewApplicationResponse,
    _body_style='bare',
    _out_message_name='NewApplicationResponse')
def NewApplicationRequest(
        self, NewApplicationRequest):
    binary_data, request_code = get_binary_data(self.udc)
    message_id = self.udc.in_smev_header.MessageId
    result = LipetskNewApplicationProxy(
        NewApplicationRequest, binary_data, request_code, message_id).process()
    return NewApplicationResponse(RegionalId=str(result))


# Получение данных Заявления для изменения
# GetApplication
@rpc(GetApplicationRequest,
     _returns=GetApplicationResponse,
     _body_style='bare',
     _out_message_name='GetApplicationResponse')
def GetApplicationRequest(ctx, request):

    ExternalId = request.ExternalId
    try:
        decl = decl_api.get_decl_by_external_id(ExternalId)
    except ApiException as exc:
        raise SpyneException(exc.code, exc.message)

    decl_proxy = LipetskDeclaredPersonDataProxy(decl)
    app_proxy = LipetskApplicantDataProxy(decl)
    dou_proxy = LipetskEduOrganizationsDataProxy(decl)
    return GetApplicationProxy(
        decl, decl_proxy, app_proxy, dou_proxy).process()


# - Изменение Заявления в региональной системе предоставления услуг
@rpc(
    UpdateApplicationRequest,
    _returns=UpdateApplicationResponse,
    _body_style='bare',
    _out_message_name='UpdateApplicationResponse')
def UpdateApplicationRequest(
        ctx, request):
    ExternalId = request.ExternalId
    try:
        decl = decl_api.get_decl_by_external_id(ExternalId)
    except ApiException as exc:
        raise SpyneException(exc.code, exc.message)

    binary_data, request_code = get_binary_data(ctx.udc)

    case_number_tag = ctx.udc.in_smev_message_document.find(
        "{http://smev.gosuslugi.ru/rev120315}CaseNumber")
    if case_number_tag is None or not case_number_tag.text:
        raise SpyneException(
            message=u"Тэг CaseNumber отсутствует или не заполнен")

    proxy_name = "reset" if request.State == "11" else "lipetsk_proxy"
    update_proxy = cont.get("UpdateProxy", proxy_name)
    return update_proxy.process(
        request, decl, binary_data=binary_data, request_code=request_code,
        case_number=case_number_tag.text)


@rpc(Unicode(nillable=False, min_occurs=1),
     DeclarantPersonal,
     ChildPersonal.customize(nillable=False, min_occurs=1),
     _in_variable_names={
         "okato": "OKATO",
         "declarant_personal": "DeclarantPersonal",
         "child_personal": "ChildPersonal"},
     _in_message_name="CheckDeclarationDOURequest",
     _returns=Array(DeclarationStatus))
def CheckDeclarationDOU(ctx, okato, declarant_personal, child_personal):

    results = []

    try:
        declarations = helpers.find_declarations(
            child_personal, True, okato
        )
    except DeclCheckException as e:
        err_code = e.message
        raise SpyneException(
            code=err_code,
            message=DeclCheckErrors.ERR_CODES[err_code][1]
        )

    for declaration in declarations:

        results.extend(
            RegionalCheckDeclarationProcess(
                declaration).process())

    return results
